@@ -42,58 +42,35 @@ def aIXb(X, a, b):
 
 def f_r1(w, X, Y, size_u, size_v):
     """Objective function for the rank-one FIR model"""
-    u, v = w[:size_u], w[size_u:size_u + size_v]
-    res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel()
+    u, v, bias = w[:size_u], w[size_u:size_u + size_v], w[size_u + size_v:]
+    res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel() - bias
     cost = .5 * linalg.norm(res) ** 2
     cost -= .5 * (linalg.norm(u) ** 2)
     return cost
 
 
 def fprime(w, X, Y, size_u, size_v):
-    u, v = w[:size_u], w[size_u:size_u + size_v]
-    res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel()
-    grad = np.empty((size_u + size_v))
+    u, v, bias = w[:size_u], w[size_u:size_u + size_v], w[size_u + size_v:]
+    res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel() - bias
+    grad = np.empty((size_u + size_v + 1))
     grad[:size_u] = IaXb(X, v, res).ravel() + u
     grad[size_u:size_u + size_v] = aIXb(X, u, res).ravel()
+    grad[size_u + size_v:] = np.sum(res)
     return - grad
 
 def f_grad(w, X, Y, size_u, size_v):
     """Returns function AND gradient"""
-    u, v = w[:size_u], w[size_u:size_u + size_v]
-    res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel()
+    u, v, bias = w[:size_u], w[size_u:size_u + size_v], w[size_u + size_v:]
+    assert len(bias) == 1
+    res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel() - bias
     cost = .5 * linalg.norm(res) ** 2
     cost -= .5 * (linalg.norm(u) ** 2)
-    grad = np.empty((size_u + size_v))
+    grad = np.empty((size_u + size_v + 1))
     grad[:size_u] = IaXb(X, v, res).ravel() + u
     grad[size_u:size_u + size_v] = aIXb(X, u, res).ravel()
+    grad[size_u + size_v:] = np.sum(res)
     return cost, -grad
 
-def f_cpd_obj(w, A, B, y, size_u):
-    """
-    .5 * ||y - A^T beta * B^T h||^2
-    """
-    h, beta = w[:size_u], w[size_u:]
-    A_beta = A.T.dot(beta)
-    B_h = B.T.dot(h)
-    residual = y - A_beta * B_h
-    cost = .5 * (residual * residual).sum()
-    cost -= .5 * (linalg.norm(u) ** 2)
-    return cost
-
-def f_cpd_grad(w, (R, A, B), y, size_u, size_v):
-    """Return function AND gradient"""
-    h, beta = w[:size_u], w[size_u:size_u+size_v]
-    A_beta = A.T.dot(beta)
-    B_h = B.T.dot(h)
-    residual = y - R.dot(A_beta * B_h)
-    cost = .5 * linalg.norm(residual) ** 2
-    cost -= .5 * (linalg.norm(h) ** 2)
-    R_residual = R.T.dot(residual)
-    grad_beta = - A.dot(B_h * R_residual)
-    grad_h = - B.dot(A_beta * R_residual) - h
-    grad = np.concatenate((grad_h, grad_beta))
-    return cost, grad
-
 def f_separate(w, X, Y, size_u, size_v, X_all):
     # for the GLM with separate designs
     u, v, z = w[:size_u], w[size_u:size_u + size_v], w[size_u + size_v:]
@@ -154,27 +131,32 @@ def f_grad_separate(w, X, Y, size_u, size_v):
     return norm, -grad
 
 def rank_one(X, y_i, n_basis,  w_i=None, callback=None, maxiter=100,
-             method='L-BFGS-B', ref_hrf=None,
-            rtol=1e-6,  verbose=0, mode='r1glm',
-            cpd=None):
+             method='L-BFGS-B', rtol=1e-6,  verbose=0, mode='r1glm'):
     """
     Run a rank-one model with a given design matrix
 
     Parameters
     ----------
     X : array-like
-        Design matrix
+        Design matrix.
 
     y_i: array-like
-        BOLD signal
+        BOLD signal.
 
-    size_u : int
-        size of the HRF
+    n_basis : int
+        Number of basis elements in the HRF.
 
     w_i : array-like
-        initial point
+        initial point.
 
     method: {'L-BFGS-B', 'TNC'}
+
+    Returns
+    -------
+    U : array
+        Estimated HRFs
+    V : array
+        Estimated activation coefficients
     """
     y_i = np.array(y_i)
     n_task = y_i.shape[1]
@@ -185,15 +167,8 @@ def rank_one(X, y_i, n_basis,  w_i=None, callback=None, maxiter=100,
         warnings.warn('Matrix X is not in sparse format. This method might be slow')
     if w_i is None:
         if mode == 'r1glm':
-            w_i = np.zeros((n_basis + size_v, n_task))
-            if sparse.issparse(X):
-                X_tmp = X.toarray()
-            else:
-                X_tmp = X
-            u0, v0 = utils.glms_from_glm(
-                X_tmp, np.eye(n_basis), ref_hrf, 1, False, y_i)
-            w_i[:n_basis] = u0.mean(1)
-            w_i[n_basis:] = v0
+            # XXX initialization
+            w_i = np.ones((n_basis + size_v + 1, n_task))
         elif mode == 'r1glms':
             w_i = np.random.randn(n_basis + 2 * size_v, n_task)
 
@@ -211,10 +186,7 @@ def rank_one(X, y_i, n_basis,  w_i=None, callback=None, maxiter=100,
         X = (Xi, Xi_all)
         ofunc = f_grad_separate
     else:
-        if cpd is None:
-            ofunc = f_grad
-        else:
-            ofunc = f_cpd_grad
+        ofunc = f_grad
 
 
     bounds = [(-1, 1)] * n_basis + [(None, None)] * (w_i.shape[0] - n_basis)
@@ -227,8 +199,6 @@ def rank_one(X, y_i, n_basis,  w_i=None, callback=None, maxiter=100,
 
     for i in range(n_task):
         args = [X, y_i[:, i], n_basis, size_v]
-        if cpd is not None:
-            args[0] = cpd
         options = {'maxiter': maxiter}
         if int(verbose) > 1:
             options['disp'] = 5
@@ -243,7 +213,7 @@ def rank_one(X, y_i, n_basis,  w_i=None, callback=None, maxiter=100,
         if verbose > 1 and not out[-1]['warnflag'] != 0:
             warnings.warn(out[-1]['task'])
         if int(verbose) > 0:
-            if ((i+1) % (n_task // 10)) == 0:
+            if ((i+1) % 500) == 0:
                 print('.. completed %s out of %s ..' % (i + 1, n_task))
             if verbose > 1:
                 if hasattr(out, 'nit'):
@@ -258,24 +228,14 @@ def rank_one(X, y_i, n_basis,  w_i=None, callback=None, maxiter=100,
 
     U = np.array(U).T
     V = np.array(V).T
-    if ref_hrf is None:
-        raise ValueError('Need a reference HRF')
-    sign = np.sign(U.T.dot(ref_hrf))
-    U *= sign
-    V *= sign
-    norm = U.max(0)  # (U.max(0) - U.min(0)) # norming twice ?
-    U = U / norm
-    V = V * norm
-
     return U, V
 
 
-def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
-        hrf_length=20, oversample=20, ref_hrf='spm',
-        rtol=1e-8, verbose=False, maxiter=500, callback=None,
-        method='L-BFGS-B', n_jobs=1, init='auto',
-        return_design_matrix=False,
-        return_raw_U=False, cache=False):
+def glm(conditions, onsets, TR, Y, basis='3hrf', mode='r1glm',
+        hrf_length=20, oversample=20, 
+        rtol=1e-8, verbose=False, maxiter=100, callback=None,
+        method='L-BFGS-B', n_jobs=1, 
+        return_design_matrix=False):
     """
     Perform a GLM from BOLD signal, given the conditons, onset,
     TR (repetition time of the scanner) and the BOLD signal.
@@ -291,8 +251,17 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
     basis:
 
         - hrf: single element basis
-        - dhrf: basis with 3 elements
-        - fir: basis with 20 elements (in multiples of TR)
+        - 3hrf: basis with 3 elements
+        - fir: basis with hrf_length elements (in multiples of TR)
+
+    **Note** the output parameters need are not normalized. 
+    Rank-1 models are specified up to a constant 
+    term between the betas and the HRF. This implies that some
+    normalization must be done prior to interpreting the activation
+    coefficients. Typically the HRF is normalized to 
+    have unit amplitude and to correlate positively with a 
+    reference HRF.
+
 
     Parameters
     ----------
@@ -313,9 +282,6 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
     mode: {'r1glm', 'r1glms', 'glms', 'glm'}
         Different GLM models.
 
-    ref_hrf: string or callable
-        Reference HRF
-
     rtol : float
         Relative tolerance for stopping criterion.
 
@@ -345,12 +311,8 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
     dmtx: array,
         Design matrix. Only returned if return_design_matrix=True
 
-    U_raw: array
-        the raw coefficients of the HRF in terms of the basis. Only
-        returned if return_raw_U=True
-
     """
-    if not mode in ('glm', 'r1glm', 'r1glm_cpd', 'r1glms', 'glms'):
+    if not mode in ('glm', 'r1glm', 'r1glms', 'glms'):
         raise NotImplementedError
     conditions = np.asarray(conditions)
     onsets = np.asarray(onsets)
@@ -358,28 +320,11 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
         raise ValueError('array conditions and onsets should have the same size')
     Y = np.asarray(Y)
     n_scans = Y.shape[0]
-    # XXX basis tolower
-    if ref_hrf == 'spm':
-        canonical_full = hrf.spmt(np.arange(0, hrf_length, TR))
-        if basis == 'fir':
-            ref_hrf = canonical_full
-        elif basis == 'dhrf':
-            if mode in ('glm', 'glms'):
-                ref_hrf = canonical_full
-            else:
-                ref_hrf = np.array([1, 0, 0])
     verbose = int(verbose)
     if verbose > 0:
         print('.. creating design matrix ..')
-    if cache != False:
-        # XXX cache is a Memory object
-        from joblib import Memory
-        memory = Memory(cachedir='', verbose=1)
-        _create_design_matrix = memory.cache(create_design_matrix)
-    else:
-        _create_design_matrix = create_design_matrix
 
-    X_design, Q = _create_design_matrix(
+    X_design, Q = create_design_matrix(
         conditions, onsets, TR, n_scans, basis, oversample, hrf_length)
     if verbose > 0:
         print('.. done creating design matrix ..')
@@ -390,43 +335,26 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
 
     size_u = Q.shape[1]
     size_v = X_design.shape[1] // size_u
-    U = np.zeros((size_u, n_task))
-    V = np.zeros((size_v, n_task))
 
-    if init in ('auto', 'glms') and mode.startswith('r1glm'):
-        if verbose > 0:
-            print('.. computing initialization ..')
-        # XXX basis
-        if mode == 'r1glms':
-            U_init, V_init, Z_init = utils.glms_from_glm(
-                X_design, Q, canonical_full, n_jobs, True, Y
-            )
-        else:
-            U_init, V_init = utils.glms_from_glm(
-                X_design, Q, canonical_full, n_jobs, False, Y)
-
-        U_init = U_init.mean(1)
-        U_init /= np.sqrt((U_init * U_init).sum(0))
-        U_init = Q.T.dot(U_init)
-        if mode == 'r1glms':
-            W_init = np.concatenate((U_init, V_init, Z_init))
-        else:
-            W_init = np.concatenate((U_init, V_init))
-    elif mode.startswith('glms'):
-        # XXX init glm
-        W_init = np.random.randn(size_u + 2 * size_v, n_task)
-    else:
-        W_init = np.random.randn(size_u + size_v)
-
-
-    cpd = None  # XXX remove
     if mode == 'glms':
         U, V = utils.glms_from_glm(
-            X_design, Q, ref_hrf, n_jobs, False, Y)
+            X_design, Q, n_jobs, False, Y)
     elif mode == 'glm':
         U, V = utils.glm(
-            X_design, Q, Y, hrf_function=ref_hrf, convolve=False)
+            X_design, Q, Y, convolve=False)
     elif mode in ('r1glm', 'r1glms'):
+        U = np.zeros((size_u, n_task))
+        V = np.zeros((size_v, n_task))
+        if verbose > 0:
+            print('.. computing initialization ..')
+        X_design_canonical, Q_canonical = create_design_matrix(conditions, onsets, TR,
+            n_scans, [hrf.spmt], oversample, hrf_length)
+        X_design_canonical = np.concatenate(
+            (X_design_canonical, np.ones((n_scans, 1))), axis=1)
+        V_init = linalg.lstsq(X_design_canonical, Y)[0]
+        U_init = np.tile(linalg.lstsq(Q, Q_canonical)[0], n_task)
+        W_init = np.concatenate((U_init, V_init))
+
         if n_jobs == -1:
             n_jobs = cpu_count()
         Y_split = np.array_split(Y, n_jobs, axis=1)
@@ -436,8 +364,7 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
         out = Parallel(n_jobs=n_jobs)(
             delayed(rank_one)(
                 X_design, y_i, size_u, w_i, callback=callback, maxiter=maxiter,
-                method=method, rtol=rtol, verbose=verbose, mode=mode,
-                cpd=cpd, ref_hrf=ref_hrf)
+                method=method, rtol=rtol, verbose=verbose, mode=mode)
             for y_i, w_i in zip(Y_split, W_init_split))
 
         counter = 0
@@ -451,37 +378,18 @@ def glm(conditions, onsets, TR, Y, basis='dhrf', mode='r1glm',
                 counter += 1
 
         raw_U = U.copy()
-        U = Q.dot(U)
         # normalize
-    norm = np.abs(U).max(0)
-    U /= norm
-    V *= norm
+    if mode == 'r1glm' and basis == '3hrf':
+        xx = np.linspace(0, hrf_length * TR)
+        generated_hrfs = U[0] * hrf.spmt(xx)[:, None] + \
+            U[1] * hrf.dspmt(xx)[:, None] + U[2] * hrf.ddspmt(xx)[:, None]
+        sign = np.sign(np.dot(generated_hrfs.T, hrf.spmt(xx)))
+        norm = np.abs(generated_hrfs).max(0)
+        U = U * sign / norm
+        V = V * sign * norm
     out = [U, V]
     if return_design_matrix:
         out.append(
             np.rec.fromarrays(list(X_design.T),
                 names=",".join(np.unique(conditions))))
-    if return_raw_U:
-        out.append(raw_U)
     return out
-
-
-if __name__ == '__main__':
-    size_u, size_v = 9, 48
-    X = sparse.csr_matrix(np.random.randn(100, size_u * size_v))
-    Z = np.random.randn(1000, 20)
-    u_true, v_true = np.random.rand(size_u, 2), 1 + .1 * np.random.randn(size_v, 2)
-    B = np.dot(u_true, v_true.T)
-    y = X.dot(B.ravel('F')) + .1 * np.random.randn(X.shape[0])
-    #y = np.array([i * y for i in range(1, 3)]).T
-    u, v = rank_one(X.A, y, size_u, np.random.randn(size_u),
-                    verbose=True, rtol=1e-10)
-
-    import pylab as plt
-    plt.matshow(B)
-    plt.title('Groud truth')
-    plt.colorbar()
-    plt.matshow(np.dot(u[:, :1], v[:, :1].T))
-    plt.title('Estimated')
-    plt.colorbar()
-    plt.show()
